#!/usr/bin/python

# This code is in the public domain.

# Step 1: Query three VO services, broadcast the result via SAMP
# (requires: astropy, pyvo, and a little helper module, vohelper)

# Queries are configured as triples of short name, access url (as from a
# registry query) query.  You *could* use TAP_SCHEMA to automate query
# generation, but that's left as an exercise to the reader

import sys
import vohelper
import pyvo


# Note that it's of course silly to use TAP to do just cone searches.
# Imagine more interesting queries here.
QUERIES = [
  ("twomass", "http://dc.zah.uni-heidelberg.de/tap",
    """SELECT TOP 1000000 raj2000, dej2000, jmag, hmag, kmag 
        FROM twomass.data
        WHERE 1=CONTAINS(
                      POINT('ICRS', raj2000, dej2000),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),
  ("allwise", "http://tapvizier.u-strasbg.fr/TAPVizieR/tap",
    """SELECT raj2000, dej2000, w1mag, w2mag, w3mag, w4mag
        FROM "II/328/allwise"
        WHERE 1=CONTAINS(
                      POINT('ICRS', raj2000, dej2000),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),
  ("sdss", "http://gea.esac.esa.int/tap-server/tap",
    """SELECT ra, dec, u_mag, g_mag, r_mag, i_mag, z_mag
        FROM gaiadr1.sdssdr9_original_valid
        WHERE 1=CONTAINS(
                      POINT('ICRS', ra, dec),
                      CIRCLE('ICRS', {ra}, {dec}, {radius}))"""),]

def main():
  # arguments: ra, dec, and sr; fill in a known-good default
  if len(sys.argv)!=4:
    ra, dec, radius = 30, 10, 0.05
  else:
    ra, dec, radius = [float(a) for a in sys.argv[1:]]


  # make (and close when done) a SAMP connection so we can
  # talk to other clients
  with vohelper.SAMP_conn() as conn:
    # see we have TOPCAT on the bus and get its client id for later use
    topcat_id = vohelper.find_client(conn, "topcat")

    # now run the three queries, sending the results via samp:
    for short_name, access_url, query in QUERIES:
      service = pyvo.dal.TAPService(access_url)
      # you could now figure out interesting things about the service,
      # e.g., its table schema and such, to potentially construct queries.
      result = service.run_sync(query.format(**locals()), maxrec=90000)
      vohelper.send_table_to(conn, topcat_id, result.table, short_name)


if __name__=="__main__":
  main()
